#!/bin/bash
#

usage()
{
cat <<EOF
Usage: `basename ${0}` <product_dir> [tar]
EOF
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
  cc=cc
  cxx=c++
  if [ ${OS1} = "Darwin" ]; then
    # Need to build using Apple Clang personality with libc++
    use_apple_clang=true
    cc=/usr/bin/clang
    cxx=/usr/bin/clang++
    cxxflags=-stdlib=libc++
  else # Linux
    if ver_greater 4.8.0 $(gcc -dumpversion); then
      # Need special GCC that understands --static-stdlibc++ and
      # --static-libgcc.
      local gccprod=$(latest_version -t ${product_dir} --exclude 7.1.0 --exclude 7.2.0 --exclude 7.3.0 --ge 6.1.0 gcc -z "${product_dir}" -z "${PRODUCTS}")
      [[ -n "${gccprod}" ]] || \
        ssi_die "On systems with GCC < 4.8.0, need access to GCC >= 6.1. Best version found was ${best_version}"
      eval setup -B ${gccprod} -z "${product_dir}" -z "${PRODUCTS}" || \
        ssi_die "Unable to set up ${gccprod}"
      # Store the details of the GCC we used so we can duplicate the
      # conditions when we use this package as an SDK.
      mkdir -p "${pkgdir}/etc"
      echo "${gccprod}" > "${pkgdir}/etc/gcc_dep"
      (( want_self_contained_build = 1 ))
      cc=gcc
      cxx=g++
    fi
  fi
}

unpack_and_patch()
{
    mkdir -p ${sourcedir}
    # unpack
    llvmtar=${tardir}/clang-${origpkgdotver}.tar.bz2
    if [[ -d "${sourcedir}/llvm-${origpkgdotver}" ]]; then
      echo "clang source already exists."
    fi
    cd ${sourcedir}
    [[ -d llvm ]] || ( tar -xf ${tardir}/llvm-${origpkgdotver}.src.tar.xz && mv llvm-${origpkgdotver}.src llvm )
    [[ -d llvm/tools/clang ]] || ( tar -xf ${tardir}/cfe-${origpkgdotver}.src.tar.xz && mv cfe-${origpkgdotver}.src llvm/tools/clang )
    [[ -d llvm/tools/clang/tools/extra ]] || ( tar -xf ${tardir}/clang-tools-extra-${origpkgdotver}.src.tar.xz && mv clang-tools-extra-${origpkgdotver}.src llvm/tools/clang/tools/extra )
    [[ -d llvm/projects/openmp ]] || ( tar -xf  ${tardir}/openmp-${origpkgdotver}.src.tar.xz && mv openmp-${origpkgdotver}.src llvm/projects/openmp )
    [[ -d llvm/projects/compiler-rt ]] || ( tar -xf  ${tardir}/compiler-rt-${origpkgdotver}.src.tar.xz && mv compiler-rt-${origpkgdotver}.src llvm/projects/compiler-rt )
    [[ -d llvm/projects/libcxx ]] || ( tar -xf ${tardir}/libcxx-${origpkgdotver}.src.tar.xz && mv libcxx-${origpkgdotver}.src llvm/projects/libcxx )
    [[ -d llvm/projects/libcxxabi ]] || ( tar -xf ${tardir}/libcxxabi-${origpkgdotver}.src.tar.xz && mv libcxxabi-${origpkgdotver}.src llvm/projects/libcxxabi )
    [[ "${OS1}" = Darwin ]] || [[ -d llvm/projects/libunwind ]] ||  \
      ( tar -xf  ${tardir}/libunwind-${pkgdotver}.src.tar.xz && mv libunwind-${pkgdotver}.src llvm/projects/libunwind )
    [[ -d llvm/tools/polly ]] ||  ( tar -xf  ${tardir}/polly-${origpkgdotver}.src.tar.xz && mv polly-${origpkgdotver}.src llvm/tools/polly )
    cd llvm/tools/clang
    patchdir=${product_dir}/${package}/${pkgver}/patch
    for patch in ${patchdir}/clang-env-var-gcc-toolchain.patch; do
      patch ${DEBUG_PATCHES:+--verbose} -p1 < "${patch}" || ssi_die "application of patch ${patch} failed."
    done
    (( ${DEBUG_PATCHES:+1} )) && exit 1
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
if [[ "${basequal}" == tar ]]; then
  maketar=${basequal}
  unset basequal
else
  maketar=${3}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=clang
origpkgver=v6_0_1
origpkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgver=${origpkgver//-/_}
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v1_04_07

get_this_dir

get_ssibuildshims

source define_basics NO_QUAL

set_qualifiers

echo "building ${package} ${pkgver} ${fullqual:+-q ${fullqual}} for ${OS}-${plat} (flavor ${flvr})"

if [ ${OS1} = "Darwin" ]
then
  prefix="${pkgdir}/Toolchains/LLVM${origpkgdotver%rc*}.xctoolchain/usr"
  echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"
  export DYLD_LIBRARY_PATH=${prefix}/lib64:${prefix}/lib
  shext=dylib
  extra_opts=(-DLLVM_CREATE_XCODE_TOOLCHAIN:BOOL=ON -DLLVM_ENABLE_LTO:STRING=Full)
  # Install in a way which is compatible with acting as a toolchain for
  # XCode.
  install_target=install-xcode-toolchain
else
  prefix="${pkgdir}"
  echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
  export LD_LIBRARY_PATH=${prefix}/lib64:${prefix}/lib
  shext=so
  install_target=install
  if [[ "$OS" == "slf6"* ]]; then
    source setup_python "${product_dir}" || ssi_die "Unable to set up required Python"
    # Would be needed if we built lldb.
    #    extra_opts+=(-DLLDB_RELOCATABLE_PYTHON=ON \
      #    -DPYTHON_LIBRARY="${PYTHON_LIB}" \
      #    -DPYTHON_INCLUDE_DIR="${PYTHON_INCLUDE}" \
      #    )
  fi
  plugin=$(${cc} -print-file-name=plugin)/include
  [[ -f "${plugin}/plugin-api.h" ]] && \
    extra_opts=(-DLLVM_BINUTILS_INCDIR=${plugin} \
    -DLLVM_ENABLE_LTO:STRING=Full)
fi

if [ "${maketar}" = "tar" ] && [ -d ${prefix}/lib ]
then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    NO_QUAL
   exit $?
fi

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

[[ -d "${sourcedir}" ]] || unpack_and_patch

source setup_cmake || ssi_die "Unable to set up required CMake"

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

set -x

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual:-NO_QUAL}

setup -B ${package} ${pkgver} ${fullqual:+-q ${fullqual}} -z ${fakedb}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# remove /usr/local/bin and . from path
PATH=`dropit -e /usr/local/bin .`
# remove /usr/local/X from LD_LIBRARY PATH and DYLD_LIBRARY_PATH
LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+$(dropit -a -p "$LD_LIBRARY_PATH" /usr/local/)}
DYLD_LIBRARY_PATH=${DYLD_LIBRARY_PATH:+$(dropit -a -p "$DYLD_LIBRARY_PATH" /usr/local/)}
# make sure libraries are in the path
export PATH=${pkgdir}/bin:${PATH}
export COMPILER_PATH=${pkgdir}/bin
# need proper environment
echo ""
echo "CC $cc CXX $cxx ${cxxflags:+cxxflags ${cxxflags}}"
echo "LD_LIBRARY_PATH $LD_LIBRARY_PATH"
echo "DYLD_LIBRARY_PATH $DYLD_LIBRARY_PATH"

cd ${blddir}
env CC=${cc} CXX=${cxx} ${cxxflags:+CXXFLAGS=${cxxflags}} \
  ${want_self_contained_build:+LDFLAGS="-static-libstdc++ -static-libgcc"} \
  cmake ../../source/llvm -DCMAKE_INSTALL_PREFIX=${pkgdir} \
  -DCMAKE_BUILD_TYPE=Release \
  -DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
  -DLLVM_TARGETS_TO_BUILD="X86" \
  -DLLVM_PARALLEL_COMPILE_JOBS:STRING=${num_cores} \
  -DLLVM_PARALLEL_LINK_JOBS:STRING=${num_cores} \
  -DLLVM_OPTIMIZED_TABLEGEN=ON \
  -DLLVM_INCLUDE_DOCS=OFF \
  -DLLVM_ENABLE_RTTI=ON \
  -DLLVM_ENABLE_EH=ON \
  -DLLVM_INSTALL_UTILS=ON \
  -DWITH_POLLY=ON \
  -DLINK_POLLY_INTO_TOOLS=ON \
  -DLIBOMP_ARCH=x86_64 \
  -DLLVM_BUILD_EXTERNAL_COMPILER_RT=ON \
  -DENABLE_LIBCXX=ON \
  -DCLANG_DEFAULT_CXX_STDLIB=libc++ \
  "${extra_opts[@]}" \
  || \
  ssi_die "Unable to configure clang."

make -j "${num_cores}" || ssi_die "Unable to make clang."
make -j "${num_cores}" ${install_target} || ssi_die "Unable to install clang."

set +x

if [ ! -d ${prefix}/bin ]
then
   echo "ERROR: failed to create ${prefix}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${prefix}/lib ]
then
   echo "ERROR: failed to create ${prefix}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} NO_QUAL

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver} ${fullqual:+-q ${fullqual}}"
source `ups setup -z "${product_dir}" -z "${PRODUCTS}" ${package} ${pkgver} ${fullqual:+-q ${fullqual}}`
echo "${package} ${pkgver} ${fullqual:+-q ${fullqual}} is installed at ${CLANG_FQ_DIR}"

if [ "${maketar}" = "tar" ]; then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    NO_QUAL
fi

exit 0
